<?php
/**
 * Administration class
 * 
 * @package Page Generator Pro
 * @author  Tim Carr
 * @version 1.0.0
 */
class Page_Generator_Pro_Admin {

    /**
     * Holds the class object.
     *
     * @since   1.1.3
     *
     * @var     object
     */
    public static $instance;

    /**
     * Holds the base object.
     *
     * @since   1.2.1
     *
     * @var     object
     */
    public $base;

    /**
     * Constructor
     * 
     * @since   1.0.0
     */
    public function __construct() {

        // Admin CSS, JS and Menu
        add_action( 'admin_enqueue_scripts', array( $this, 'admin_scripts_css' ) );
        add_action( 'admin_menu', array( $this, 'admin_menu' ), 8 );
        add_action( 'parent_file', array( $this, 'admin_menu_hierarchy_correction' ), 999 );

        // Run any row actions called from the WP_List_Table
        add_action( 'init', array( $this, 'run_keyword_table_row_actions' ) );

        // Localization
        add_action( 'plugins_loaded', array( $this, 'load_language_files' ) );

    }

    /**
     * Enqueues CSS and JS
     *
     * @since   1.0.0
     */
    public function admin_scripts_css() {

        global $post;

        // Get base instance
        $this->base = ( class_exists( 'Page_Generator_Pro' ) ? Page_Generator_Pro::get_instance() : Page_Generator::get_instance() );

        // CSS - always load
        wp_enqueue_style( $this->base->plugin->name . '-admin', $this->base->plugin->url . 'assets/css/admin.css', array(), $this->base->plugin->version );

        // JS - register scripts we might use
        wp_register_script( $this->base->plugin->name . '-generate-content', $this->base->plugin->url . 'assets/js/min/generate-content-min.js', array( 'jquery' ), $this->base->plugin->version, true );
        wp_register_script( $this->base->plugin->name . '-modal', $this->base->plugin->url . 'assets/js/min/modal-min.js', array( 'jquery' ), $this->base->plugin->version, true );
        wp_register_script( $this->base->plugin->name . '-selectize', $this->base->plugin->url . 'assets/js/min/selectize-min.js', array( 'jquery' ), $this->base->plugin->version, true );
        wp_register_script( $this->base->plugin->name . '-synchronous-ajax', $this->base->plugin->url . 'assets/js/min/synchronous-ajax-min.js', array( 'jquery' ), $this->base->plugin->version, true );
        
        // Determine the screen that we're on
        $screen = $this->get_screen();

        // If we're not on a plugin screen, exit
        if ( ! $screen['screen'] ) {
            return;
        }

        // If here, we're on a plugin screen
        // Conditionally load scripts and styles depending on which section of the Plugin we're loading
        switch ( $screen['screen'] ) {
            /**
             * Content: Groups
             */
            case 'content_groups':
                // JS: Plugin
                wp_enqueue_script( $this->base->plugin->name . '-generate-content' );
                wp_enqueue_script( $this->base->plugin->name . '-modal' );

                // Get localization strings
                $localization = Page_Generator_Pro_Groups::get_instance()->get_titles_and_messages();

                // Add data to localization depending on the screen we're viewing
                switch ( $screen['section'] ) {
                    /**
                     * Content: Groups: Add / WP_List_Table
                     */
                    case 'wp_list_table':
                        break;

                    /**
                     * Content: Groups: Edit
                     */
                    case 'edit':
                        // Prevents errors with meta boxes and Yoast
                        wp_enqueue_media();

                        // CSS: WP Zinc
                        wp_enqueue_style( 'wpzinc-admin-selectize' );

                        // JS: WP Zinc
                        wp_enqueue_script( 'jquery-ui-sortable' );
                        wp_enqueue_script( 'wpzinc-admin-selectize' );
                        wp_enqueue_script( 'wpzinc-admin-tags' );
                        
                        // JS: Plugin
                        wp_enqueue_script( $this->base->plugin->name . '-selectize' ); 

                        // Get localization strings
                        $localization['post_id'] = ( isset( $post->ID ) ? $post->ID : false );
                        $localization['keywords'] = Page_Generator_Pro_Keywords::get_instance()->get_column( 'keyword' );
                        
                        break;
                }

                // Apply Localization
                wp_localize_script( $this->base->plugin->name . '-generate-content', 'page_generator_pro_generate_content', $localization );
                break;

            /**
             * Generate
             */
            case 'generate':
                wp_enqueue_script( 'jquery-ui-progressbar' );
                wp_enqueue_script( $this->base->plugin->name . '-synchronous-ajax' );
                break;
        }

        // CSS
        if ( class_exists( 'Page_Generator' ) ) {
            // Hide 'Add New' if a Group exists
            $number_of_groups = Page_Generator_Pro_Groups::get_instance()->get_count();
            if ( $number_of_groups > 0 ) {
                ?>
                <style type="text/css">body.post-type-page-generator-pro a.page-title-action { display: none; }</style>
                <?php
            }
        }
        
    }

    /**
     * Returns a string denoting the current Plugin screen that the user is on.
     *
     * @since   1.9.4
     *
     * @return  array   Screen and Section (if false, we're not on this Plugin's screens)
     */
    public function get_screen() {

        // Assume we're not on a plugin screen
        $result = array(
            'screen'  => false,
            'section' => false,
        );

        // Bail if we can't determine this
        if ( ! function_exists( 'get_current_screen' ) ) {
            return $result;
        }

        // Get screen
        $screen = get_current_screen();

        // Bail if we're not on any Plugin screen
        if ( strpos( $screen->base, $this->base->plugin->name ) === false && $screen->post_type != Page_Generator_Pro_PostType::get_instance()->post_type_name ) {
            return $result;
        }

        switch ( $screen->id ) {
            /**
             * Settings
             */
            case $this->base->plugin->name . '_page_' . $this->base->plugin->name . '-settings':
                return array(
                    'screen'    => 'settings',
                    'section'   => 'settings',
                );
                break;

            /**
             * Keywords
             */
            case $this->base->plugin->name . '_page_' . $this->base->plugin->name . '-keywords':
                $cmd = ( isset( $_REQUEST['cmd'] ) ? sanitize_text_field( $_REQUEST['cmd'] ) : false );
                switch ( $cmd ) {
                    // Keywords WP_List_Table
                    case false:
                        return array(
                            'screen'    => 'keywords',
                            'section'   => 'wp_list_table',
                        );
                        break;

                    // Add/Edit
                    case 'form':
                        return array(
                            'screen'    => 'keywords',
                            'section'   => 'edit',
                        );
                        break;

                    // Generate Locations
                    case 'form-locations':
                        return array(
                            'screen'    => 'keywords',
                            'section'   => 'generate_locations',
                        );
                        break;

                    // Generate Phone Area Codes
                    case 'form-phone':
                        return array(
                            'screen'    => 'keywords',
                            'section'   => 'generate_phone_area_codes',
                        );
                        break;
                }
                break;

            /**
             * Content: Groups: Table
             */
            case 'edit-' . $this->base->plugin->name:
                switch ( $screen->action ) {
                    // WP_List_Table
                    case '':
                        return array(
                            'screen'    => 'content_groups',
                            'section'   => 'wp_list_table',
                        );
                        break;
                }
                break;

            /**
             * Content: Groups: Add/Edit
             */
            case $this->base->plugin->name:
            case 'page-generator-pro':
                return array(
                    'screen'    => 'content_groups',
                    'section'   => 'edit',
                );
                break;

            /**
             * Content: Terms: Add/Edit
             */
            case 'edit-page-generator-tax':
                switch ( $screen->base ) {
                    // WP_List_Table
                    case 'edit-tags':
                        return array(
                            'screen'    => 'content_terms',
                            'section'   => 'wp_list_table',
                        );
                        break;

                    // Edit
                    case 'term':
                        return array(
                            'screen'    => 'content_terms',
                            'section'   => 'edit',
                        );
                        break;
                }
                break;

            /**
             * Content: Generate
             */
            case $this->base->plugin->name . '_page_' . $this->base->plugin->name . '-generate':
                return array(
                    'screen'    => 'generate',
                    'section'   => 'generate',
                );
                break;

        }

        // If here, we couldn't determine the screen
        return $result;

    }

    /**
     * Add the Plugin to the WordPress Administration Menu
     *
     * @since   1.0.0
     */
    public function admin_menu() {

        global $submenu;
        
        // Get base instance
        $this->base = ( class_exists( 'Page_Generator_Pro' ) ? Page_Generator_Pro::get_instance() : Page_Generator::get_instance() );

        // Main Menu
        add_menu_page( $this->base->plugin->displayName, $this->base->plugin->displayName, 'manage_options', $this->base->plugin->name . '-keywords', array( $this, 'keywords_screen' ), 'dashicons-format-aside' );

        // Sub Menu
        $keywords_page = add_submenu_page( $this->base->plugin->name . '-keywords', __( 'Keywords', $this->base->plugin->name ), __( 'Keywords', $this->base->plugin->name ), 'manage_options', $this->base->plugin->name . '-keywords', array( $this, 'keywords_screen' ) );    
        $groups_page   = add_submenu_page( $this->base->plugin->name . '-keywords', __( 'Generate', $this->base->plugin->name ), __( 'Generate', $this->base->plugin->name ), 'manage_options', 'edit.php?post_type=' . Page_Generator_Pro_PostType::get_instance()->post_type_name );    
        $generate_page = add_submenu_page( $this->base->plugin->name . '-keywords', __( 'Generate', $this->base->plugin->name ), __( 'Generate', $this->base->plugin->name ), 'manage_options', $this->base->plugin->name . '-generate', array( $this, 'generate_screen' ) );    

        // Menus
        $upgrade_page = add_submenu_page( $this->base->plugin->name . '-keywords', __( 'Upgrade', $this->base->plugin->name ), __( 'Upgrade', $this->base->plugin->name ), 'manage_options', $this->base->plugin->name . '-upgrade', array( $this, 'upgrade_screen' ) );

    }

    /**
     * Ensures this Plugin's top level Admin menu remains open when the user clicks on:
     * - Generate Content
     * - Generate Terms
     *
     * This prevents the 'wrong' admin menu being open (e.g. Posts)
     *
     * @since   1.2.3
     *
     * @param   string  $parent_file    Parent Admin Menu File Name
     * @return  string                  Parent Admin Menu File Name
     */
    public function admin_menu_hierarchy_correction( $parent_file ) {

        global $current_screen;

        // If we're creating or editing a Content Group, set the $parent_file to this Plugin's registered menu name
        if ( $current_screen->base == 'post' && $current_screen->post_type == Page_Generator_Pro_PostType::get_instance()->post_type_name ) {
            // The free version uses a different top level filename
            if ( class_exists( 'Page_Generator' ) ) {
                return $this->base->plugin->name . '-keywords';
            }

            return $this->base->plugin->name;
        }

        return $parent_file;

    }

    /**
     * Run any row actions on the Keywords WP_List_Table now
     *
     * @since   1.2.3
     */
    public function run_keyword_table_row_actions() {

        // Get base instance
        $this->base = ( class_exists( 'Page_Generator_Pro' ) ? Page_Generator_Pro::get_instance() : Page_Generator::get_instance() );

        // Get Page and Command
        $page = ( isset( $_REQUEST['page'] ) ? sanitize_text_field( $_REQUEST['page'] ) : '' );
        $cmd = ( ( isset($_GET['cmd'] ) ) ? sanitize_text_field( $_GET['cmd'] ) : false );
        if ( ! $cmd ) {
            if ( isset( $_POST['action'] ) ) {
                $cmd = sanitize_text_field( $_POST['action'] );
            }
            if ( isset( $_POST['action2'] ) ) {
                $cmd = sanitize_text_field( $_POST['action2'] );
            }  
        }

        // If we're not on the keywords table, bail
        if ( empty( $page ) || $page != $this->base->plugin->name . '-keywords' ) {
            return;
        }

        // If no command, bail
        if ( ! $cmd ) {
            return;
        }
        
        switch ( $cmd ) {

            /**
             * Delete Keyword
             */
            case 'delete':
                // Setup notices class, enabling persistent storage
                $notices = Page_Generator_Pro_Notices::get_instance();
                $notices->enable_store();
                $notices->set_key_prefix( 'page_generator_pro_groups_notices_' . wp_get_current_user()->ID );

                // Delete keywords
                $result = $this->delete_keywords();
                if ( is_string( $result ) ) {
                    // Add error message and redirect back to the keyword table
                    $notices->add_error_notice( $result );
                } elseif ( $result === true ) {
                    // Success
                    $notices->add_success_notice( __( 'Keyword(s) deleted successfully.', $this->base->plugin->name ) );
                }
                break; 

            /**
             * Delete Keyword
             */
            case '-1':
                // Bail if no keywords were selected
                if ( ! isset( $_POST['ids'] ) || empty( $_POST['ids'] ) ) {
                    return;
                }  

                // Setup notices class, enabling persistent storage
                $notices = Page_Generator_Pro_Notices::get_instance();
                $notices->enable_store();
                $notices->set_key_prefix( 'page_generator_pro_groups_notices_' . wp_get_current_user()->ID );

                // Delete keywords
                $result = $this->delete_keywords();
                if ( is_string( $result ) ) {
                    // Add error message and redirect back to the keyword table
                    $notices->add_error_notice( $result );
                } elseif ( $result === true ) {
                    // Success
                    $notices->add_success_notice( __( 'Keyword(s) deleted successfully.', $this->base->plugin->name ) );
                }
                break;

            /**
             * Don't process any other commands
             */
            default:
                return;
                break;

        }

        wp_redirect( 'admin.php?page=' . $this->base->plugin->name . '-keywords' );
        die();

    }

    /**
     * Outputs the Keywords Screens
     *
     * @since 1.0.0
     */
    public function keywords_screen() {

        // Setup notices class
        $notices = Page_Generator_Pro_Notices::get_instance();
        $notices->set_key_prefix( 'page_generator_pro_groups_notices_' . wp_get_current_user()->ID );

        // Get Page
        $page = ( isset( $_REQUEST['page'] ) ? sanitize_text_field( $_REQUEST['page'] ) : '' );

        // Get command
        $cmd = ( ( isset($_GET['cmd'] ) ) ? sanitize_text_field( $_GET['cmd'] ) : '' );
        switch ( $cmd ) {

            /**
             * Add / Edit Keyword
             */
            case 'form':
                // Get keyword from POST data or DB
                if ( isset( $_POST['keyword'] ) ) {
                    // Get keyword from POST data
                    $keyword = stripslashes_deep( $_POST );
                } else if ( isset( $_GET['id'] ) ) {
                    // Get keyword from DB
                    $keyword = Page_Generator_Pro_Keywords::get_instance()->get_by_id( $_GET['id'] );
                }

                // Save keyword
                $result = $this->save_keyword();
                if ( is_wp_error( $result ) ) {
                    $notices->add_error_notice( $result->get_error_message() );
                } else if ( is_numeric( $result ) ) {
                    $notices->add_success_notice( __( 'Keyword saved successfully.', $this->base->plugin->name ) );

                    // Fetch saved keyword from DB
                    $keyword = Page_Generator_Pro_Keywords::get_instance()->get_by_id( absint( $result ) );
                }

                // View
                $view = 'views/admin/keywords-form.php';
                
                break;

            /**
             * Index Table
             */
            case 'delete':
            default: 
                // View
                $view = 'views/admin/keywords-table.php';
                
                break;   

        }

        // Load View
        include_once( $this->base->plugin->folder . $view ); 

    }

    /**
     * Save Keyword
     *
     * @since   1.0.0
     *
     * @return  mixed   WP_Error | int
     */
    public function save_keyword() {

        // Check if a POST request was made
        if ( ! isset( $_POST['submit'] ) ) {
            return false;
        }

        // Run security checks
        // Missing nonce 
        if ( ! isset( $_POST[ $this->base->plugin->name . '_nonce' ] ) ) { 
            return new WP_Error( __( 'Nonce field is missing. Settings NOT saved.', $this->base->plugin->name ) );
        }

        // Invalid nonce
        if ( ! wp_verify_nonce( $_POST[ $this->base->plugin->name . '_nonce' ], 'save_keyword' ) ) {
            return new WP_Error( __( 'Invalid nonce specified. Settings NOT saved.', $this->base->plugin->name ) );
        }

        // Get ID
        $id = ( ( isset($_REQUEST['id'] ) && ! empty( $_REQUEST['id'] ) ) ? $_REQUEST['id'] : '' );

        // Get keywords instance
        $instance = Page_Generator_Pro_Keywords::get_instance();

        // Build keyword
        $keyword = array(
            'keyword'   => sanitize_text_field( $_POST['keyword'] ),
            'data'      => $instance->import_text_file_data( $_POST['data'] ),
        );

        // Save Keyword
        return $instance->save( $keyword, $id );

    }

    /**
     * Delete Keyword(s), if commands to do this have been sent in the request
     *
     * @since   1.0.0
     *
     * @return  mixed   WP_Error | true
     */
    public function delete_keywords() {

        // Check an action exists
        if ( ! isset( $_POST['action'] ) && ! isset( $_POST['action2'] ) && ! isset( $_GET['cmd'] ) ) {
            return false;
        }

        // Set flag
        $deleted = false;

        // Get instance
        $instance = Page_Generator_Pro_Keywords::get_instance();

        // Bulk Delete
        if ( ( isset( $_POST['action2'] ) && $_POST['action2'] != '-1' ) ||
             ( isset( $_POST['action'] ) && $_POST['action'] != '-1' ) ) {

            $result = $instance->delete( $_POST['ids'] );
            if ( is_wp_error( $result ) ) {
                return $result;
            }

            $deleted = true;

        }

        // Single Delete
        if ( isset( $_GET['cmd'] ) && $_GET['cmd'] == 'delete' ) {
            $result = $instance->delete( $_GET['id'] );
            if ( is_wp_error( $result ) ) {
                return $result;
            }

            $deleted = true;

        }

        // If a delete happened, return true
        return $deleted;

    }

    /**
     * Generates content for the given Group and Group Type
     *
     * @since   1.2.3
     */
    public function generate_screen() {

        // Get base instance
        $this->base = ( class_exists( 'Page_Generator_Pro' ) ? Page_Generator_Pro::get_instance() : Page_Generator::get_instance() );

        // Check a Group ID and Type was supplied
        if ( ! isset( $_REQUEST['id'] ) ) {
            return;
        }
        $id = absint( $_REQUEST['id'] );
        $type = ( isset( $_REQUEST['type'] ) ? sanitize_text_field( $_REQUEST['type'] ) : 'content' );

        // Get group settings and return to Group URL, depending on the type
        switch ( $type ) {
            case 'content':
            default:
                $settings = Page_Generator_Pro_Groups::get_instance()->get_settings( $id );
                $return_url = admin_url( 'post.php?post=' . $id . '&amp;action=edit' );
                break;
        }
        
        // Bail if no settings
        if ( ! $settings ) {
            return;
        }

        // Calculate how many pages could be generated
        $number_of_pages_to_generate = Page_Generator_Pro_Generate::get_instance()->get_max_number_of_pages( $settings );
          
        // Check that the number of posts doesn't exceed the maximum that can be generated
        if ( $settings['numberOfPosts'] > $number_of_pages_to_generate ) {
            $settings['numberOfPosts'] = $number_of_pages_to_generate;
        }  

        // If no limit specified, set one now
        if ( $settings['numberOfPosts'] == 0 ) {
            if ( $settings['method'] == 'random' ) {
                $settings['numberOfPosts'] = 10;
            } else {
                $settings['numberOfPosts'] = $number_of_pages_to_generate;
            }
        }

        // Code which executes the process is in the view, as we use sync JS
        $view = 'views/admin/generate-run.php';

        // Load View
        include_once( $this->base->plugin->folder . $view );

    }

    /**
     * Loads plugin textdomain
     *
     * @since   1.0.0
     */
    public function load_language_files() {

        // Get base instance
        $this->base = ( class_exists( 'Page_Generator_Pro' ) ? Page_Generator_Pro::get_instance() : Page_Generator::get_instance() );

        load_plugin_textdomain( $this->base->plugin->name, false, $this->base->plugin->name . '/languages/' );

    } 

    /**
     * Returns the singleton instance of the class.
     *
     * @since   1.1.3
     *
     * @return  object Class.
     */
    public static function get_instance() {

        if ( ! isset( self::$instance ) && ! ( self::$instance instanceof self ) ) {
            self::$instance = new self;
        }

        return self::$instance;

    }

}

// Load the class
$page_generator_pro_admin = Page_Generator_Pro_Admin::get_instance();