<?php
/**
 * AJAX class
 * 
 * @package Page Generator Pro
 * @author  Tim Carr
 * @version 1.0.0
 */
class Page_Generator_Pro_AJAX {

    /**
     * Holds the class object.
     *
     * @since   1.1.3
     *
     * @var     object
     */
    public static $instance;

    /**
     * Constructor
     *
     * @since   1.0.0
     */
    public function __construct() {

        // Generate: Authors
        add_action( 'wp_ajax_page_generator_pro_search_authors', array( $this, 'search_authors' ) );

        // Generate
        add_action( 'wp_ajax_page_generator_pro_generate', array( $this, 'generate' ) ); // Backward compat < 1.6.0
        add_action( 'wp_ajax_page_generator_pro_generate_content', array( $this, 'generate_content' ) );
        
    }

    /**
     * Searches for Authors for the given freeform text
     *
     * @since   1.8.3
     */
    public function search_authors() {

        // Get vars
        $query = sanitize_text_field( $_REQUEST['query'] );

        // Get results
        $users = new WP_User_Query( array(
            'search' => '*' . $query . '*',
        ) );

        // If an error occured, bail
        if ( is_wp_error( $users ) ) {
            return wp_send_json_error( $users->get_error_message() );
        }

        // Build array
        $users_array = array();
        $results = $users->get_results();
        if ( ! empty( $results ) ) {
            foreach ( $results as $user ) {
                $users_array[] = array(
                    'id'        => $user->ID,
                    'user_login'=> $user->user_login,
                );
            }
        }

        // Done
        wp_send_json_success( $users_array );

    }

    /**
     * Backward compat function for Page Generator Pro 1.6.0 and older.
     *
     * Passes the request to generate_content()
     *
     * @since   1.0.0
     */
    public function generate() {

        $this->generate_content();

    }

    /**
     * Generates a Page, Post or CPT
     *
     * @since   1.6.1
     */
    public function generate_content() {

        // Validate
        $result = $this->generate_validation();

        // Run
        $start  = microtime( true );
        $result = Page_Generator_Pro_Generate::get_instance()->generate_content( $result['group_id'], $result['current_index'], $result['test_mode'] );
        $end    = microtime( true );

        // Return
        $this->generate_return( $result, $start, $end );

    } 

    /**
     * Runs validation when AJAX calls are made to generate content or terms,
     * returning the Group ID and Current Index.
     *
     * @since   1.6.1
     *
     * @return  array   Group ID and Current Index
     */
    private function generate_validation() {

        // Sanitize inputs
        if ( ! isset( $_POST['id'] ) ) {
            wp_send_json_error( __( 'No group ID was specified!', 'page-generator-pro' ) );
            die();
        }
        
        return array(
            'group_id'      => absint( $_POST['id'] ),
            'current_index' => ( isset( $_POST['current_index'] ) ? absint( $_POST['current_index'] ) : 0 ),
            'test_mode'     => ( isset( $_POST['test_mode'] ) ? true : false ),
        );

    }

    /**
     * Returns the generation result as a JSON error or success
     *
     * @since   1.6.1
     *
     * @param   mixed   $result     WP_Error | string URL
     * @param   float   $start      Start Execution Time
     * @param   float   $end        End Execution Time
     */
    private function generate_return( $result, $start, $end ) {

        // Return error or success JSON
        if ( is_wp_error( $result ) ) {
            wp_send_json_error( $result->get_error_code() . ': ' . $result->get_error_message() );
        }

        // If here, run routine worked
        wp_send_json_success( array(
            'url'               => $result['url'],
            'keywords_terms'    => $result['keywords_terms'],
            'start'             => $start,
            'end'               => $end,
            'duration'          => round ( ( $end - $start ), 2 ),
            'memory_usage'      => round( memory_get_usage() / 1024 / 1024 ),
            'memory_peak_usage' => round( memory_get_peak_usage() / 1024 / 1024 ),
        ) );

    }

    /**
     * Returns the singleton instance of the class.
     *
     * @since   1.1.3
     *
     * @return  object Class.
     */
    public static function get_instance() {

        if ( ! isset( self::$instance ) && ! ( self::$instance instanceof self ) ) {
            self::$instance = new self;
        }

        return self::$instance;

    }

}

// Load the class
$page_generator_pro_ajax = Page_Generator_Pro_AJAX::get_instance();