<?php
/**
 * Generate class. Handles generating Pages, Posts, Custom Post Types
 * and Taxonomy Terms.
 * 
 * @package Page_Generator_Pro
 * @author  Tim Carr
 * @version 1.0.0
 */
class Page_Generator_Pro_Generate {

    /**
     * Holds the class object.
     *
     * @since   1.1.3
     *
     * @var     object
     */
    public static $instance;

    /**
     * Holds the array of found keywords across all settings.
     *
     * @since   1.2.0
     *
     * @var     array
     */
    public $required_keywords = array();

    /**
     * Holds the array of found keywords that require a fixed term across all settings.
     *
     * @since   1.2.0
     *
     * @var     array
     */
    public $required_keywords_fixed = array();

    /**
     * Holds the array of found keywords that require a sub term to be extracted by a given column name.
     *
     * @since   1.7.8
     *
     * @var     array
     */
    public $required_keywords_columns = array();

    /**
     * Holds the array of keywords to replace e.g. {city}
     *
     * @since   1.3.1
     *
     * @var     array
     */
    public $searches = array();

    /**
     * Holds the array of keyword values to replace e.g. Birmingham
     *
     * @since   1.3.1
     *
     * @var     array
     */
    public $replacements = array();

    /**
     * Constructor.
     *
     * @since   1.9.3
     */
    public function __construct() {

        add_action( 'page_generator_pro_generate_content_after', array( $this, 'generate_content_finished' ), 10, 2 );
        add_action( 'page_generator_pro_generate_terms_after', array( $this, 'generate_terms_finished' ), 10, 2 );

    }

    /**
     * Calculates the maximum number of items that will be generated based
     * on the settings.
     *
     * @since   1.1.5
     *
     * @param   array   $settings   Group Settings (either a Content or Term Group)
     * @return  mixed               WP_Error | integer
     */
    public function get_max_number_of_pages( $settings ) {

        // Build a class array of required keywords that need replacing with data
        $this->find_keywords_in_settings( $settings );

        // Bail if no keywords were found
        if ( count( $this->required_keywords ) == 0 ) {
            return 0;
        }

        // Get the terms for each required keyword
        $keywords = $this->get_keywords_terms_columns_delimiters( $this->required_keywords );

        // Bail if no keywords were found
        if ( empty( $keywords['terms'] ) ) {
            return 0;
        }

        $total = 0;
        foreach ( $keywords['terms'] as $keyword => $terms ) {
            if ( count( $terms ) > 0 && ( count( $terms ) < $total || $total == 0 ) ) {
                $total = count( $terms );
            }
        }

        return $total;

    }

    /**
     * Deprecated function for generating Posts, Pages and Custom Post Types
     * from a given Group.
     *
     * Use generate_content() instead.
     *
     * @since   1.0.0
     *
     * @param   int     $group_id   Group ID
     * @param   int     $index      Keyword Index
     * @param   bool    $test_mode  Test Mode
     * @return  mixed               WP_Error | array
     */
    public function generate( $group_id, $index = 0, $test_mode = false ) {

        return $this->generate_content( $group_id, $index, $test_mode );

    }
    
    /**
     * Main function to generate a Page, Post, Custom Post Type or Taxonomy Term
     *
     * @since   1.6.1
     *
     * @param   int     $group_id   Group ID
     * @param   int     $index      Keyword Index
     * @param   bool    $test_mode  Test Mode
     * @return  mixed               WP_Error | URL
     */
    public function generate_content( $group_id, $index = 0, $test_mode = false ) {

        // Performance debugging
        $start  = microtime( true );
        
        // Get instances
        $common_instance    = Page_Generator_Pro_Common::get_instance();
        $groups_instance    = Page_Generator_Pro_Groups::get_instance();

        // Get group settings
        $settings = $groups_instance->get_settings( $group_id );
        if ( ! $settings ) {
            return new WP_Error( 'group_error', sprintf( __( 'Group ID %s could not be found.', 'page-generator-pro' ), $group_id ) );
        }

        // Build a class array of required keywords that need replacing with data
        $this->find_keywords_in_settings( $settings );
        if ( count( $this->required_keywords ) == 0 ) {
            return new WP_Error( 'keyword_error', __( 'No keywords were specified in the Group.', 'page-generator-pro' ) );
        }

        // Build a keywords array comprising of terms, columns and delimiters for each of the required keywords
        $keywords = $this->get_keywords_terms_columns_delimiters( $this->required_keywords );
        if ( count( $keywords['terms'] ) == 0 ) {
            return new WP_Error( 'keyword_error', __( 'Keywords were specified in the Group, but no keywords exist in either the Keywords section of the Plugin or as a Taxonomy.', 'page-generator-pro' ) );
        }

        // Build array of keyword --> term key/value pairs to use for this generation
        $keywords_terms = $this->get_keywords_terms( $keywords, $settings['method'], $index );
        if ( is_wp_error( $keywords_terms ) ) {
            return $keywords_terms;
        }

        // Rotate Author
        if ( isset( $settings['rotateAuthors'] ) ) {
            $authors = $common_instance->get_authors();
            $user_index = ( $index % count( $authors ) );
        }

        // Iterate through each keyword and term key/value pair
        foreach ( $keywords_terms as $keyword => $term ) {
            // Build search and replacement arrays, including keyword transformations
            $this->build_search_replace_arrays( $keyword, $term, $keywords );

            // Iterate through Group Settings, replacing keywords with terms
            array_walk_recursive( $settings, array( $this, 'replace_keywords_in_array' ) );
        }

        // Determine the Post Parent
        $post_parent = ( ( isset( $settings['pageParent'] ) && isset( $settings['pageParent'][ $settings['type'] ] ) && ! empty( $settings['pageParent'][ $settings['type'] ] ) ) ? $settings['pageParent'][ $settings['type'] ] : 0 );
        if ( ! is_numeric( $post_parent ) ) {
            // Find the Post ID based on the given name
            $parent = get_page_by_path( $post_parent, OBJECT, $settings['type'] );

            if ( ! $parent ) {
                $post_parent = 0;
            } else {
                $post_parent = $parent->ID;
            }
        }

        // Build Post args
        $post_args = array(
            'post_type'     => $settings['type'],
            'post_title'    => $settings['title'],
            'post_content'  => $settings['content'],
            'post_excerpt'  => $settings['excerpt'],
            'post_status'   => ( $test_mode ? 'draft' : $settings['status'] ),
            'post_author'   => ( ( isset( $settings['rotateAuthors'] ) && $settings['rotateAuthors'] == 1 ) ? $authors[ $user_index ]->ID : $settings['author'] ), // ID
            'comment_status'=> ( ( isset( $settings['comments'] ) && $settings['comments'] == 1 ) ? 'open' : 'closed' ),
            'ping_status'   => ( ( isset( $settings['trackbacks'] ) && $settings['trackbacks'] == 1 ) ? 'open' : 'closed' ),
            'post_parent'   => $post_parent,
        );

        // Define Post Name
        // If no Permalink exists, use the Post Title
        if ( ! empty( $settings['permalink'] ) ) {
            $post_args['post_name'] = sanitize_title( $settings['permalink'] );
        } else {
            $post_args['post_name'] = sanitize_title( $post_args['post_title'] );
        }

        // Define the Post Date
        $post_args['post_date'] = date_i18n( 'Y-m-d H:i:s' );

        /**
         * Filters arguments used for creating or updating a Post when running
         * content generation.
         *
         * @since   1.6.1
         *
         * @param   array   $post_args  wp_insert_post() / wp_update_post() compatible arguments
         * @param   array   $settings   Content Group Settings
         */
        $post_args = apply_filters( 'page_generator_pro_generate_post_args', $post_args, $settings );

        // Create Page
        $post_id = wp_insert_post( $post_args, true );

        // Check Post creation / update worked
        if ( is_wp_error( $post_id ) ) {
            // UTF-8 encode the Title, Excerpt and Content
            $post_args['post_title'] = utf8_encode( $post_args['post_title'] );
            $post_args['post_excerpt'] = utf8_encode( $post_args['post_excerpt'] );
            $post_args['post_content'] = utf8_encode( $post_args['post_content'] );
            
            // Try again
            if ( count( $existing_post->posts ) > 0 ) {
                // Update Page, Post or CPT
                $post_id = wp_update_post( $post_args, true ); 
            } else {
                // Create Page, Post or CPT
                $post_id = wp_insert_post( $post_args, true ); 
            }
        }

        // If Post creation / update still didn't work, bail
        if ( is_wp_error( $post_id ) ) {
            $post_id->add_data( $post_args, $post_id->get_error_code() );
            return $post_id;
        }

        // Store this Generation ID in the Post's meta, so we can edit/delete the generated Post(s) in the future
        update_post_meta( $post_id, '_page_generator_pro_group', $group_id );

        // Post / Page Template
        // Backward compat for Free
        if ( ! empty( $settings['pageTemplate'] ) && ! is_array( $settings['pageTemplate'] ) ) {
            update_post_meta( $post_id, '_wp_page_template', $settings['pageTemplate'] );
        }
        if ( ! empty( $settings['pageTemplate'][ $settings['type'] ] ) ) {
            update_post_meta( $post_id, '_wp_page_template', $settings['pageTemplate'][ $settings['type'] ] );
        }

        // Get URL of Page/Post/CPT just generated
        $url = get_bloginfo( 'url' ) . '?page_id=' . $post_id . '&preview=true';

        // Request that the user review the plugin. Notification displayed later,
        // can be called multiple times and won't re-display the notification if dismissed.
        if ( ! $test_mode ) {
            if ( class_exists( 'Page_Generator' ) ) {
                Page_Generator::get_instance()->dashboard->request_review();    
            } else {
                Page_Generator_Pro::get_instance()->dashboard->request_review();
            }
        }

        // Performance debugging
        $end = microtime( true );

        // Return the URL and keyword / term replacements used
        return array(
            'url'           => $url,
            'keywords_terms'=> $keywords_terms,

            // Performance data
            'start'             => $start,
            'end'               => $end,
            'duration'          => round ( ( $end - $start ), 2 ),
            'memory_usage'      => round( memory_get_usage() / 1024 / 1024 ),
            'memory_peak_usage' => round( memory_get_peak_usage() / 1024 / 1024 ),
        );

    }

    /**
     * Adds keyword transformations to the class $searches and $replacements arrays,
     * based on the given $keyword (search) and $term (replacement).
     *
     * Also includes transformations (e.g. uppercase_all, lowercase_all etc)
     *
     * @since   1.7.8
     *
     * @param   string  $keyword    Keyword (search)
     * @param   string  $term       Term (replace)
     * @param   array   $keywords   Keywords and Terms array
     */
    private function build_search_replace_arrays( $keyword, $term, $keywords ) {

        // Reset search and replacement arrays for this keyword
        $this->searches     = array();
        $this->replacements = array();

        // Start the arrays with the basic search and replace
        $this->searches[] = '{' . $keyword . '}';
        $this->replacements[] = $term;
            
    }

    /**
     * Recursively goes through the settings array, finding any {keywords}
     * specified, to build up an array of keywords we need to fetch.
     *
     * @since   1.0.0
     *
     * @param   array   $settings   Settings
     */
    private function find_keywords_in_settings( $settings ) {

        // Get all keywords
        $keywords = Page_Generator_Pro_Keywords::get_instance()->get_all( 'keyword', 'ASC', -1 );

        // Recursively walk through all settings to find all keywords
        array_walk_recursive( $settings, array( $this, 'find_keywords_in_string' ) );

        // The class arrays will now store keywords, keywords requiring transformations and keywords with column names
        // Itereate through each to remove any duplicates
        if ( count( $this->required_keywords_fixed ) > 0 ) {
            foreach ( $this->required_keywords_fixed as $keyword => $columns ) {
                $this->required_keywords_fixed[ $keyword ] = array_values( array_unique( $columns ) );
            }
        }
        if ( count( $this->required_keywords_columns ) > 0 ) {
            foreach ( $this->required_keywords_columns as $keyword => $columns ) {
                $this->required_keywords_columns[ $keyword ] = array_values( array_unique( $columns ) );
            }
        }

    }

    /**
     * For the given array of keywords, only returns keywords with terms, column names and delimiters
     * where each keywords have terms.
     *
     * @since   1.6.5
     *
     * @param   array   $required_keywords  Required Keywords
     * @return  array                       Keywords with Terms, Columns and Delimiters
     */
    private function get_keywords_terms_columns_delimiters( $required_keywords ) {

        // Get keywords instance
        $keywords_instance = Page_Generator_Pro_Keywords::get_instance();

        // Define blank array for keywords with terms and keywords with columns
        $results = array(
            'terms'     => array(),
            'columns'   => array(),
            'delimiters'=> array(),
        );

        foreach ( $required_keywords as $key => $keyword ) {

            // Get terms for this keyword
            // If this keyword starts with 'taxonomy_', try to fetch the terms for the Taxonomy
            if ( strpos( $keyword, 'taxonomy_') !== false && strpos( $keyword, 'taxonomy_' ) == 0 ) {
                $result = get_terms( array(
                    'taxonomy'              => str_replace( 'taxonomy_', '', $keyword ),
                    'hide_empty'            => false,
                    'fields'                => 'names',
                    'update_term_meta_cache'=> false,
                ) );

                // Skip if no results
                if ( ! is_array( $result ) ) {
                    continue;
                }
                if ( count( $result ) == 0 ) {
                    continue;
                }

                $results['terms'][ $keyword ] = $result;
            } else {
                $result = $keywords_instance->get_by( 'keyword', $keyword );

                // Skip if no results
                if ( ! is_array( $result ) ) {
                    continue;
                }
                if ( count( $result ) == 0 ) {
                    continue;
                }

                $results['terms'][ $keyword ]       = $result['dataArr'];
                $results['columns'][ $keyword ]     = $result['columnsArr'];
                $results['delimiters'][ $keyword ]  = $result['delimiter'];
            }
        }

        // Return results
        return $results;

    }

    /**
     * Returns an array of keyword and term key / value pairs.
     * 
     * @param   array   $keywords   Keyword and Terms
     * @param   string  $method     Generation Method
     * @param   int     $index      Generation Index
     * @return  mixed               WP_Error | array
     * */
    private function get_keywords_terms( $keywords, $method, $index ) {

        $keywords_terms = array();
        foreach ( $keywords['terms'] as $keyword => $terms ) {
            // Use modulo to get the term index for this keyword
            $term_index = ( $index % count( $terms ) );   

            // Build the keyword => term key/value pairs
            $keywords_terms[ $keyword ] = $terms[ $term_index ];
        }

        // Cleanup the terms
        foreach ( $keywords_terms as $key => $term ) {
            $keywords_terms[ $key ] = trim( html_entity_decode( $term ) );
        }

        return $keywords_terms;

    }

    /**
     * Performs a search on the given string to find any {keywords}
     *
     * @since 1.2.0
     *
     * @param   string  $content    Array Value (string to search)
     * @param   string  $key        Array Key
     */
    private function find_keywords_in_string( $content, $key ) {

        // If $content is an object, iterate this call
        if ( is_object( $content ) ) {
            return array_walk_recursive( $content, array( $this, 'find_keywords_in_string' ) );
        }

        // Get keywords and spins
        preg_match_all( "|{(.+?)}|", $content, $matches );

        // Continue if no matches found
        if ( ! is_array( $matches ) ) {
            return;
        }
        if ( count( $matches[1] ) == 0 ) {
            return;
        }

        // Iterate through matches
        foreach ( $matches[1] as $m_key => $keyword ) {
            // Unset some vars
            unset( $transformation, $column );

            // Ignore spins
            if ( strpos( $keyword, "|" ) !== false ) {
                continue;
            }

            // If a keyword is within spintax at the start of the string (e.g. {{service}|{service2}} ),
            // we get an additional leading curly brace for some reason.  Remove it
            $keyword = str_replace( '{', '', $keyword );
            $keyword = str_replace( '}', '', $keyword );

            // If there's a transformation flag applied to the keyword, extract and remove it
            if ( strpos( $keyword, ':' ) !== false ) {
                list( $keyword, $transformation ) = explode( ':', $keyword );
            }

            // If there's a column name applied to the keyword, extract and remove it
            $keyword_column_start_bracket_position = strpos( $keyword, '(' );
            $keyword_column_end_bracket_position = strpos( $keyword, ')', $keyword_column_start_bracket_position );
            if ( $keyword_column_start_bracket_position !== false && $keyword_column_end_bracket_position !== false ) {
                // Extract
                $column = substr( $keyword, ( $keyword_column_start_bracket_position + 1 ), ( $keyword_column_end_bracket_position - $keyword_column_start_bracket_position - 1 ) );

                // Remove
                $keyword = preg_replace( '/\(.*?\)/', '', $keyword );
            }

            // Lowercase keyword, to avoid duplicates e.g. {City} and {city}
            $keyword = strtolower( $keyword );

            // If this keyword is not in our required_keywords array, add it
            if ( ! in_array( $keyword, $this->required_keywords ) ) {
                $this->required_keywords[ $keyword ] = $keyword;
            }

            // If the transformation is a number, add this keyword to the required keywords fixed array
            if ( isset( $transformation ) && is_numeric( $transformation ) ) {
                if ( ! isset( $this->required_keywords_fixed[ $keyword ] ) || ! is_array( $this->required_keywords_fixed[ $keyword ] ) ) {
                    $this->required_keywords_fixed[ $keyword ] = array(
                        $transformation,
                    );
                } else {
                    $this->required_keywords_fixed[ $keyword ][] = $transformation;
                }
            }

            // If the column is specified, add this keyword to the required keywords columns array
            if ( isset( $column ) && ! empty( $column ) ) {
                if ( ! isset( $this->required_keywords_columns[ $keyword ] ) || ! is_array( $this->required_keywords_columns[ $keyword ] ) ) {
                    $this->required_keywords_columns[ $keyword ] = array(
                        $column,
                    );
                } else {
                    $this->required_keywords_columns[ $keyword ][] = $column;
                }
            } 
        }

    }

    /**
     * array_walk_recursive callback, which finds $this->searches, replacing with
     * $this->replacements in $item.
     *
     * Also performs spintax.
     *
     * @since   1.3.1
     *
     * @param   mixed   $item   Item (array, object, string)
     * @param   string  $key    Key
     */
    private function replace_keywords_in_array( &$item, $key ) {

        // If the settings key's value is an array, walk through it recursively to search/replace
        // Otherwise do a standard search/replace on the string
        if ( is_array( $item ) ) {
            // Array
            array_walk_recursive( $item, array( $this, 'replace_keywords_in_array' ) );
        } elseif( is_object( $item ) ) {
            // Object
            array_walk_recursive( $item, array( $this, 'replace_keywords_in_array' ) );
        }

        // If here, we have a string
        // Perform keyword replacement now
        $item = str_ireplace( $this->searches, $this->replacements, $item ); 

    }

    /**
     * Runs any actions once content generation has completed.
     *
     * @since   1.9.3
     *
     * @param   int     $group_id   Group ID
     * @param   bool    $test_mode  Test Mode
     */
    public function generate_content_finished( $group_id, $test_mode ) {

        include_once( ABSPATH . 'wp-admin/includes/plugin.php' );

        // Elementor: Clear Cache
        if ( is_plugin_active( 'elementor/elementor.php' ) ) {
            Elementor\Plugin::$instance->files_manager->clear_cache();
        }

    }

    /**
     * Returns all Post IDs generated by the given Group ID
     *
     * @since   1.9.1
     *
     * @param   int     $group_id   Group ID
     * @return  mixed               WP_Error | array
     */
    private function get_generated_content_post_ids( $group_id ) {

        // Get all Posts
        $posts = new WP_Query( array (
            'post_type'     => 'any',
            'post_status'   => 'publish',
            'posts_per_page'=> -1,
            'meta_query'    => array(
                array(
                    'key'   => '_page_generator_pro_group',
                    'value' => absint( $group_id ),
                ),
            ),
            'fields'        => 'ids',
        ) );

        // If no Posts found, return false, as there's nothing to delete
        if ( count( $posts->posts ) == 0 ) {
            return new WP_Error( 'page_generator_pro_generate_get_generated_content_post_ids', __( 'No content has been generated by this group.', 'page-generator-pro' ) );
        }

        // Return Post IDs
        return $posts->posts;

    }

    /**
     * Returns the singleton instance of the class.
     *
     * @since   1.1.3
     *
     * @return  object Class.
     */
    public static function get_instance() {

        if ( ! isset( self::$instance ) && ! ( self::$instance instanceof self ) ) {
            self::$instance = new self;
        }

        return self::$instance;

    }

}